function [Labels,Labels_Pass1,idxModeDist] = SpatialSpectralDiffusionLearningActive(X,K,DistStruct,Centers,I,J,M,N,WindowSize,pct,GT,SampleMethod,PeakOpts, G_iterated)

% Compute a clustering by diffusion mode estimation.
% Regularize by spatial consensus.

% This differs from SpatialSpectralDiffusionLearning only its use of active learning to
% request labels for the pixels that are prediced to be the most
% challenging to learn.

GT=UniqueGT(GT);
Labels=zeros(1,size(X,1));
NeededActive=floor(size(X,1)*pct); % Number of pixels needed for active learning

%% In case that the spatial window is radius 0, just do usual diffusion learning

if WindowSize<1
    Labels=FSFclustering(X,K,DistStruct,Centers);
end

%% Load relevant variables

Density=DistStruct.Density;

%% If no modes are passed in, compute them.

if isempty(Centers)
    
    PWdist=squareform(pdist(X));
    
    [DeltaDensity, Density] = FindDensityPeaks(X,K,PWdist,opts);
    
    [~,idx]=sort(DeltaDensity,'descend');
    
    Centers(1)=idx(1);
    
    Needed=K-1;
    start=2;
    
    % This is a technical script to make sure the cores that are learned are
    % not duplicates.
    
    while Needed>0
        if abs(DeltaDensity(idx(start))-DeltaDensity(Centers(K-Needed)))>1e-6
            Centers(end+1)=idx(start);
            Needed=Needed-1;
        end
        start=start+1;
    end
end

%% Assign labels to centers

for k=1:K
    Labels(Centers(k))=k;
end

%% For principled active learning, compute the gap between a point's
%  distance to its nearest two modes, then assign ground truth labels
%  to the points that are close to equidistance between two modes.

G=DistStruct.G;
t=PeakOpts.DiffusionTime;
WeightedEigVecs=(G.EigenVals.^t)'.*G.EigenVecs;

if strcmp(SampleMethod,'Principled')
    
    if strcmp(PeakOpts.ModeDetection,'Euclidean')
        DistMode=pdist2(X,X(Centers,:));
    elseif strcmp(PeakOpts.ModeDetection,'Diffusion')
        DistMode=pdist2(WeightedEigVecs,WeightedEigVecs(Centers,:));
    end
    
    DistModeSorted=sort(DistMode,2);
    
    %% Assign labels to the points whose distance to nearest class mode is most ambiguous
    
    [~,idxModeDist]=sort(DistModeSorted(:,2)-DistModeSorted(:,1),'ascend');
    
    j=1;
    
    while NeededActive>0
        MatchingMode=find(GT(Centers)==GT(idxModeDist(j)));
        if length(MatchingMode)==1 && GT(idxModeDist(j))~=0
            Labels(idxModeDist(j))=Labels(Centers(MatchingMode));
            NeededActive=NeededActive-1;
        end
        j=j+1;
    end
end

%% For active learning with random samples, simply pick random GT points at start of labeling.

if strcmp(SampleMethod,'Random')
    
    while NeededActive>0
        idx=randsample(size(X,1),1);
        MatchingMode=find(GT(Centers)==GT(idx));
        if length(MatchingMode)==1 && GT(idx)~=0 && Labels(idx)==0
            Labels(idx)=Labels(Centers(MatchingMode));
            NeededActive=NeededActive-1;
        end
    end
    idxModeDist=[];
end

%% Sort points by density

[~,idx]=sort(Density,'descend');

%% First pass

LabelsMatrix=full(sparse(I,J,Labels,M,N));

for j=1:length(Labels)
    if Labels(idx(j))==0
        NN=FindNearestLabeledNeighbor(X,j,idx,G,G_iterated,PeakOpts);
        NN=find(idx==NN);
        Labels(idx(j))=Labels(idx(NN));
        % Special case when many value have same density
        if Labels(idx(j))==0
            temp=find(Labels>0);
            if strcmp(PeakOpts.ModeDetection,'Euclidean')
                PWdist=pdist2(X(idx(j),:),X(temp,:));
            elseif strcmp(PeakOpts.ModeDetection,'Diffusion')
                PWdist=pdist2(WeightedEigVecs(idx(j),:),WeightedEigVecs(temp,:));
            end
            [~,NN]=min(PWdist);
            Labels(idx(j))=Labels(temp(NN));

               
           
        end
        % Check spatial consensus label.  IF there is disagreement, wait
        % until second pass to label.
        LabelsMatrix(I(idx(j)),J(idx(j)))=Labels(idx(j));
        SpatialConsensusLabel=SpatialConsensus(LabelsMatrix,idx(j),M,N,I,J,WindowSize);
        
        if ~(SpatialConsensusLabel==Labels(idx(j))) && (SpatialConsensusLabel>0)
            Labels(idx(j))=0;
            LabelsMatrix(I(idx(j)),J(idx(j)))=0;
        end
    end
end

%% Save incomplete labels from first pass before moving to second pass
Labels_Pass1=Labels;

%% Second pass

for j=1:length(Labels)
    if Labels(idx(j))==0
        NN=FindNearestLabeledNeighbor(X,j,idx,G,G_iterated,PeakOpts);
        NN=find(idx==NN);
        Labels(idx(j))=Labels(idx(NN));
        % Special case when many values have same density
        if Labels(idx(j))==0
            temp=find(Labels>0);
            if strcmp(PeakOpts.ModeDetection,'Euclidean')
                PWdist=pdist2(X(idx(j),:),X(temp,:));
            elseif strcmp(PeakOpts.ModeDetection,'Diffusion')
                PWdist=pdist2(WeightedEigVecs(idx(j),:),WeightedEigVecs(temp,:));
            end
            [~,NN]=min(PWdist);
            Labels(idx(j))=Labels(temp(NN));
        end
        
        % Enforce spatial consensus if obvious
        LabelsMatrix=sparse(I(idx(1:j)),J(idx(1:j)),Labels(idx(1:j)),M,N);
        [SpatialConsensusLabel,ConsensusFrac]=SpatialConsensus(LabelsMatrix,idx(j),M,N,I,J,WindowSize);
        
        if ConsensusFrac>.5
            Labels(idx(j))=SpatialConsensusLabel;
        end
    end
end

end

